// MessController.java                              MM 2014
// MVC-Controller-Grundstruktur

import javax.swing.*;                        // JOptionPane
import java.awt.event.*;     // WindowAdapter, Listener,...

/**
 * Controller-Grundstruktur,
 * installiert und deinstalliert Controller,
 * als Ereignisverarbeiter.
 */
public class MessController
  extends WindowAdapter
  implements ActionListener, KeyListener
{
/* ----------------------------------------------------- */
                                                     // MVC
/**
 * Mathematisches Modell,
 * enthaelt Funktionalitaet des Problems.
 */
  private MessModel model;

/**
 * Zum Controller gehoeriger View.
 */
  private MessView view;

/* ----------------------------------------------------- */
                                        // MVC-Installation
/**
 * Konstruktor, initialisiert Model und View.
 * @param m Mathematisches Modell
 * @param v zum Controller gehoeriger View
 */
  public MessController( MessModel m, MessView v)
  {
    model = m;
    view = v;
  }

/* ----------------------------------------------------- */
                                    // handleEvent-Methoden
/**
 * ActionListener,
 * Ereignisauswertung fuer Buttons.
 */
  public void actionPerformed( ActionEvent ae)
  {
    Object command = ae.getActionCommand();

                                     // Startzustand setzen
    if( command.equals( view.ACTION_NEU))
    {
      model.reset();
      view.reset();
    }
                                        // Programm beenden
    if( command.equals( view.ACTION_ENDE))
    {
      view.release();
    }
  }

/* ----------------------------------------------------- */

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt,
 * Methode tut nichts.
 */
  public void keyPressed( KeyEvent ke)
  {
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde losgelassen.
 */
  public void keyReleased( KeyEvent ke)
  {
                              // Zahleneingabe abschliessen
    if( ke.getKeyChar() == '\n')
    {
      String zahl = ((JTextField)ke.getSource()).getText();
      if( zahl != "")
      {
        try
        {                                    // Zahl setzen
          model.addWert( Double.valueOf( zahl));
        }
        catch( Exception e)
        {
         setFehlerDialog( "Nur Zahlen eingeben!");
         view.reset();                // View zuruecksetzen
        }
      }
    }
  }

/**
 * KeyListener,
 * Ereignisauswertung Taste wurde gedrueckt
 * und losgelassen, Methode tut nichts.
 * 
 */
  public void keyTyped( KeyEvent ke)
  {
  }

/* ----------------------------------------------------- */
/**
 * Abfangen von Eingabefehlern.
 * @param text Fehlermeldung
 */
  private void setFehlerDialog( String text)
  {
    JOptionPane.showMessageDialog( view, text,
    "Fehlermeldung", JOptionPane.ERROR_MESSAGE);
  }

/* ----------------------------------------------------- */

/**
 * WindowAdapter,
 * Schliessen des Fensters.
 */
  public void windowClosing( WindowEvent we)
  {
      view.release();
  }

/* ----------------------------------------------------- */
                                  // MVC-Deinstallation
/**
 * Freigabe der Controllers,
 * setzt Model und View zurck.
 */
  public void release()
  {
    model = null;
    view = null;
  }
}
