// MessView                                         MM 2014
// MVC-View-Grundstruktur

import java.awt.*;                                 // Color
import javax.swing.*;                    // GUI-Komponenten
import java.util.*;                             // Observer

/**
 * View-Grundstruktur.
 */
public class MessView
  extends JFrame
  implements Observer
{
/* ----------------------------------------------------- */
                                          // GUI-Konstanten
/**
 * Konstante, Berechnung wiederholen.
 */
  public final String ACTION_NEU = "Neu";

/**
 * Konstante, Programm beenden.
 */
  public final String ACTION_ENDE = "Beenden";

/* ----------------------------------------------------- */
                                         // GUI-Komponenten
/**
 * Label Ergebnis der Mittelwertberechnung.
 */
  private JLabel lbErgebnis;

/**
 * Textfeld zur Eingabe der Messwerte.
 */
  private JTextField tfZahl;

/* ----------------------------------------------------- */
                                                     // MVC
/**
 * Mathematisches Modell,
 * enthaelt Funktionalitaet des Problems.
 */
  private MessModel model;

/**
 * Zum View gehoeriger Controller.
 */
  private MessController controller;

/* ----------------------------------------------------- */
                                        // MVC-Installation
/**
 * Konstruktor, setzt Ueberschrift und installiert MVC.
 * @param model Model, welches dargestellt werden soll
 * @param titel Ueberschrift
 */
  public MessView( MessModel model, String titel)
  {
// Titelzeile
    super( titel);
// Model
    this.model = model;
    this.model.addObserver( this);          // Ueberwachung
// Controller
    controller = makeController();
// View
    makeView();
  }

/**
 * Erzeugt Controller, Empfaenger fuer Ereignisse.
 * @return Controller fuer View
 */
  private MessController makeController()
  {
    return new MessController( model, this);
  }

/**
 * Erzeugt View, baut die Oberflaeche auf.
 */
  private void makeView()
  {
// ContentPane
    getContentPane().add( createMWBox());

// Startzustand
    reset();

// Fenster
    addWindowListener( controller);
    pack();
    setVisible( true);
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
                                             // ContentPane
/**
 * Senkrechte Box fuer Ein-, Ausgaben und Butten.
 */
  private Box createMWBox()
  {
    Box box = Box.createVerticalBox();

    box.add( Box.createVerticalStrut(20));
    box.add( createBoxIn());                    // Eingaben
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxOut());                   // Ausgaben
    box.add( Box.createVerticalStrut(20));
    box.add( createBoxBt());                      // Button
    box.add( Box.createVerticalStrut(20));

    return box;
  }

/**
 * Waagerechte Box fuer Eingabe,
 * Controller verarbeitet Eingaben.
 */
  private Box createBoxIn()
  {                                          // Eingabefeld
    tfZahl = new JTextField( 10);
    tfZahl.setHorizontalAlignment( JTextField.RIGHT);
    tfZahl.addKeyListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Wert:"));
    box.add( Box.createHorizontalStrut(50));
    box.add( tfZahl);

    return box;
  }

/**
 * Waagerechte Box fuer Ausgabe.
 */
  private Box createBoxOut()
  {
    lbErgebnis  = new JLabel( "0");

    Box box = Box.createHorizontalBox();

    box.add( new JLabel( "Mittelwert:"));                   // Schritt
    box.add( Box.createHorizontalStrut(20));
    box.add( lbErgebnis);

    return box;
  }

/**
 * Waagerechte Box fuer Steuerbutton,
 * Controller verarbeitet Buttonereignisse.
 */
  private Box createBoxBt()
  {
    JButton btNeu     = new JButton( ACTION_NEU);
    btNeu.addActionListener( controller);

    JButton btBeenden = new JButton( ACTION_ENDE);
    btBeenden.addActionListener( controller);

    Box box = Box.createHorizontalBox();

    box.add( btNeu);
    box.add( btBeenden);

    return box;
  }

/* ----------------------------------------------------- */
                        // service-Methoden fuer Controller
/**
 * Setzt Startzustand, Button Neu.
 */
  public void reset()
  {
// Eingabefeld
    tfZahl.setText( "");
// Ausgabefeld
    lbErgebnis.setText( "0");
  }

/* ----------------------------------------------------- */
/* ----------------------------------------------------- */
                                      // MVC-Deinstallation
/**
 * Deinstalliert MVC,
 * setzt Model und Controller zurck.
 */
  public void release()
  {
// Fenster
    dispose();
// Controller
    controller.release();
    controller = null;
// Model
    model.deleteObserver( this);
    model = null;
  }

/* ----------------------------------------------------- */
                                        // Observer-Methode
/**
 * Ueberschreibt Interfacemethode,
 * legt Reaktion auf Aenderungen fest.
 * @param m Model, welches Aenderungen meldet
 * @param o geaenderte Objekte
 */
  public void update( Observable m, Object o)
  {
    if( model != m) return;

    lbErgebnis.setText( "" + model.mittelWert());
    tfZahl.setText( "");

  }
}
